/*
 *    GS2MGSearch.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *   the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;


// Greenstone classes
import org.greenstone.gsdl3.util.*;

// XML classes
import org.w3c.dom.Document;
import org.w3c.dom.Element; 
import org.w3c.dom.NodeList;

/**
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @author <a href="mailto:mdewsnip@cs.waikato.ac.nz">Michael Dewsnip</a>
 */

public class GS2MGSearch 
    extends AbstractMGSearch 
{
    protected GDBMWrapper gdbm_src = null;

    /** constructor */
    public GS2MGSearch()
    {
	this.gdbm_src = new GDBMWrapper();
    }
    public void cleanUp() {
	super.cleanUp();
	this.gdbm_src.closeDatabase();
    }

    /** configure this service */
    public boolean configure(Element info, Element extra_info)
    {

    	// Open GDBM database for querying
	String gdbm_db_file = GSFile.GDBMDatabaseFile(this.site_home, this.cluster_name);
	if (!this.gdbm_src.openDatabase(gdbm_db_file, GDBMWrapper.READER)) {
	    System.err.println("Error: Could not open GDBM database!");
	    return false;
	}
	return super.configure(info, extra_info);
    }
    /** returns the document type of the doc that the specified node 
	belongs to. should be one of 
	GSXML.DOC_TYPE_SIMPLE, 
	GSXML.DOC_TYPE_PAGED, 
	GSXML.DOC_TYPE_HIERARCHY
    */
    protected String getDocType(String node_id){
	DBInfo info = this.gdbm_src.getInfo(node_id);
	if (info == null) {
	    return GSXML.DOC_TYPE_SIMPLE;
	}
	String doc_type = info.getInfo("doctype");
	if (!doc_type.equals("")&&!doc_type.equals("doc")) {
	    return doc_type;
	}

	String top_id = OID.getTop(node_id);
	boolean is_top = (top_id.equals(node_id) ? true : false);
	
	String children = info.getInfo("contains");
	boolean is_leaf = (children.equals("") ? true : false);

	if (is_top && is_leaf) { // a single section document
	    return GSXML.DOC_TYPE_SIMPLE;
	}

	// now we just check the top node
	if (!is_top) { // we need to look at the top info
	    info = this.gdbm_src.getInfo(top_id);
	}
	if (info == null) {
	    return GSXML.DOC_TYPE_HIERARCHY;
	}
 
	String childtype = info.getInfo("childtype");
	if (childtype.equals("Paged")) {
	    return GSXML.DOC_TYPE_PAGED;
	}
	return GSXML.DOC_TYPE_HIERARCHY;

    }
    
    /** returns true if the node has child nodes */
    protected boolean hasChildren(String node_id){
	DBInfo info = this.gdbm_src.getInfo(node_id);
	if (info == null) {
	    return false;
	}
	String contains = info.getInfo("contains");
	if (contains.equals("")) {
	    return false;
	}
	return true;
    }
    
    /** returns true if the node has a parent */
    protected boolean hasParent(String node_id){
	String parent = OID.getParent(node_id);
	if (parent.equals(node_id)) {
	    return false;
	}
	return true;
    }
    
   /** convert MG internal id to Greenstone oid */
    protected String MGNum2OID(long docnum)
    {
	return this.gdbm_src.docnum2OID(docnum);
	
    }

}


